@@ -7,27 +7,26 @@ from helix_catalog_sdk.catalog import Catalog
 from helix_catalog_sdk.repo import DirectoryRepo
 from helix_catalog_sdk.enums import HelixEnvironment
 
-
 TEST_DIR = os.path.dirname(__file__)
 SOURCE_CATALOG = os.path.join(TEST_DIR, "test_catalog")
+SOURCE_DATA = os.path.join(TEST_DIR, "test_data")
 
 
 def setup_module(module: ModuleType) -> None:
     print("Setting up")
     print(" * Copying directory 'test_catalog' to 'catalog' for testing")
-    shutil.copytree(
-        src=SOURCE_CATALOG, dst="catalog",
-    )
+    shutil.copytree(src=SOURCE_CATALOG, dst="catalog")
+    shutil.copytree(src=SOURCE_DATA, dst="data")
 
 
 def teardown_module(module: ModuleType) -> None:
     print("Tearing down")
     print(" * Removing directory 'catalog'")
     shutil.rmtree("catalog", ignore_errors=True)
+    shutil.rmtree("data", ignore_errors=True)
 
 
 def test_catalog_get_data_source() -> None:
-
     repo = DirectoryRepo(location="")
     catalog = Catalog(repo=repo)
 
@@ -47,6 +46,19 @@ def test_catalog_get_data_source() -> None:
         people_resource.full_path
         == "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092021.json"
     )
+    assert people_resource.last_processed is not None
+    assert (
+        people_resource.last_processed.get_last_processed(HelixEnvironment.PRODUCTION)
+        == "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092021.json"
+    )
+    people_resource.last_processed.set_last_processed(
+        "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092022.json",
+        HelixEnvironment.PRODUCTION,
+    )
+    assert (
+        people_resource.last_processed.get_last_processed(HelixEnvironment.PRODUCTION)
+        == "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092022.json"
+    )
 
     places_resource = data_source.resources[1]
     assert places_resource.name == "places"
@@ -80,7 +92,6 @@ def test_catalog_get_data_source() -> None:
 
 
 def test_catalog_update_data_source_resource() -> None:
-
     repo = DirectoryRepo(location="")
     catalog = Catalog(repo=repo)
     catalog.get_all_data_sources()
@@ -140,19 +151,33 @@ def test_catalog_update_data_source_resource() -> None:
     json_resources = json_data.get("resources")
     assert json_resources == [
         {
+            "last_processed": {
+                "dev": "s3://dev-ingestion/raw/gencon/icd/People/PeopleFeed_06092021.json",
+                "production": "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092021.json",
+                "qa": "",
+                "staging": "s3://staging-ingestion/raw/gencon/icd/People/PeopleFeed_06092021.json",
+            },
             "name": "people",
             "path": "People/PeopleFeed_06142021.json",
             "path_slug": "People/PeopleFeed_",
+            "date_segment": None,
+            "date_format": None,
         },
         {
+            "last_processed": None,
             "name": "places",
             "path": "Places/PlaceFeed_06172021.json",
             "path_slug": "Places/PlaceFeed_",
+            "date_segment": None,
+            "date_format": None,
         },
         {
+            "last_processed": None,
             "name": "things",
             "path": "Things/ThingFeed_06092021.json",
             "path_slug": "Things/ThingFeed_",
+            "date_segment": None,
+            "date_format": None,
         },
     ]
 
@@ -166,3 +191,55 @@ def test_catalog_update_data_source_resource_returns_none() -> None:
         "s3://prod-ingestion/raw/gencon/icd/test/does_not_exist.csv"
     )
     assert updated_data_source is None
+
+
+def test_catalog_update_resource_last_processed() -> None:
+    repo = DirectoryRepo(location="")
+    catalog = Catalog(repo=repo)
+    data_source = catalog.get_data_source(
+        "catalog/raw/gencon/icd.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    catalog.update_resource_last_processed(
+        "people",
+        "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092022.json",
+        data_source,
+        HelixEnvironment.PRODUCTION,
+    )
+    data_source = catalog.get_data_source(
+        "catalog/raw/gencon/icd.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    people_resource = data_source.resources[0]
+    assert people_resource is not None
+    last_processed = people_resource.last_processed
+    assert last_processed is not None
+    assert (
+        last_processed.get_last_processed(HelixEnvironment.PRODUCTION)
+        == "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092022.json"
+    )
+
+
+def test_catalog_get_resource_unprocessed_directories() -> None:
+    repo = DirectoryRepo(location="")
+    catalog = Catalog(repo=repo)
+    data_source = catalog.get_data_source(
+        "catalog/raw/thedacare/betteru.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    paths_to_process = catalog.get_resource_unprocessed_directories(
+        "betteru", data_source, HelixEnvironment.PRODUCTION
+    )
+    assert len(paths_to_process) == 2
+    assert paths_to_process == [
+        "data/raw/thedacare/betteru/2021-07-02",
+        "data/raw/thedacare/betteru/2021-07-05",
+    ]
+    catalog.update_resource_last_processed(
+        "betteru", paths_to_process[0], data_source, HelixEnvironment.PRODUCTION
+    )
+    paths_to_process = catalog.get_resource_unprocessed_directories(
+        "betteru", data_source, HelixEnvironment.PRODUCTION
+    )
+    assert len(paths_to_process) == 1
+    assert paths_to_process == ["data/raw/thedacare/betteru/2021-07-05"]
